// BatchLink Utility
//
// Copyright (c) 2000 Mike Woodring
// http://www.bearcanyon.com
//
// Usage:
//
// BatchLink OtherDir [/q]
//
// For every *.* in OtherDir
// {
//     Create hard link in current dir to same-named-file in OtherDir
// }
//
// The optional /q command line flag tells BatchLink to perform
// the operation without first confirming the directories and file
// counts involved.
//
#include "stdafx.h"

#ifdef UNICODE
typedef wstring _tstring;
#else
typedef string _tstring;
#endif

void ShowUsage()
{
    _tprintf(__TEXT("\nInvalid command line.\n\nUsage: BatchLink TargetDir [/s:<filespec>] [/q]\n"));
    _tprintf(__TEXT("\n"));
    _tprintf(__TEXT("Link to *.h in c:\\mssdk\\include: BatchLink c:\\mssdk\\include /s:*.h\n"));
    _tprintf(__TEXT("Link to *.* in c:\\mssdk\\include (no prompt): BatchLink c:\\mssdk\\include /q\n"));
}

BOOL VolumeSupportsHardLinks( TCHAR *pszFile )
{
    DWORD   dwVolFlags;
    BOOL    fOkay;
    TCHAR   szDrive[4];

    ZeroMemory(szDrive, sizeof(szDrive));
    _tcsncpy(szDrive, pszFile, 3);

    fOkay = GetVolumeInformation( szDrive,
                                  0, 0,
                                  0, 0, &dwVolFlags,
                                  0, 0 );

    if( !fOkay || !(dwVolFlags & FILE_SUPPORTS_REPARSE_POINTS) )
    {
        fOkay = FALSE;
    }

    return(fOkay);
}

BOOL CreateLink( TCHAR *pszLinkFile, TCHAR *pszSrcFile )
{
    TCHAR szSourceFileName[MAX_PATH];
    TCHAR szLinkFileName[MAX_PATH];
    TCHAR szCurDir[MAX_PATH];

    GetCurrentDirectory(MAX_PATH, szCurDir);

    if( _tcschr(pszLinkFile, '\\') )
    {
        _tcscpy(szLinkFileName, pszLinkFile);
    }
    else
    {
        _stprintf( szLinkFileName, __TEXT("%s%s"),
                   szCurDir, pszLinkFile );
    }

    if( _tcschr(pszSrcFile, __TEXT('\\')) )
    {
        _tcscpy(szSourceFileName, pszSrcFile);
    }
    else
    {
        _stprintf( szSourceFileName, __TEXT("%s%s"),
                   szCurDir, pszSrcFile );
    }

    if( !VolumeSupportsHardLinks(szSourceFileName) ||
        !VolumeSupportsHardLinks(szLinkFileName) )
    {
        _tprintf(__TEXT("One or both of the volumes specified does not support hard links."));
        return(FALSE);
    }

    return CreateHardLink(szLinkFileName, szSourceFileName, 0);
}

list<_tstring *>    g_listFiles;
TCHAR               g_szTargetDir[MAX_PATH];

bool ParseCmdLine( TCHAR *pszArgs[], int cArgs, TCHAR *pszTargetDir, TCHAR *pszFileSpec, bool *pfQuiet )
{
    // BatchLink TargetDir [/s:*.*] [/q]
    //
    if( cArgs < 2 )
    {
        return(false);
    }

    _tcscpy(pszTargetDir, pszArgs[1]);

    // Setup some defaults.
    //
    *pfQuiet = false;
    _tcscpy(pszFileSpec, __TEXT("*.*"));

    for( int nArg = 2; nArg < cArgs; nArg++ )
    {
        if( _tcsnicmp(__TEXT("/s:"), pszArgs[nArg], 3) == 0 )
        {
            _tcscpy(pszFileSpec, &pszArgs[nArg][3]);
        }
        else if( _tcsnicmp(__TEXT("/q"), pszArgs[nArg], 2) == 0 )
        {
            *pfQuiet = true;
        }
    }

    return(true);
}

bool IsValidFile( const WIN32_FIND_DATA *pFileInfo )
{
    bool fValid = false;

    DWORD dwAttrs = pFileInfo->dwFileAttributes;

    // A file is considered "valid" to this application if is it
    // not (a) a directory, (b) hidden, or (c) a temporary file.
    //
    if( ((dwAttrs & FILE_ATTRIBUTE_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY) &&
        ((dwAttrs & FILE_ATTRIBUTE_HIDDEN) != FILE_ATTRIBUTE_HIDDEN) &&
        ((dwAttrs & FILE_ATTRIBUTE_TEMPORARY) != FILE_ATTRIBUTE_TEMPORARY) )
    {
        fValid = true;
    }

    return(fValid);
}

void ShowError( DWORD dwError = GetLastError() )
{
    Win32Error  Err(dwError);

    _tprintf( __TEXT("Error %d (0x%08x): %s\n"),
              dwError, dwError, (TCHAR *)Err );
}

void LinkToFile( _tstring *psz )
{
    const TCHAR *pszFile = psz->c_str();
    TCHAR szSrcFile[MAX_PATH];

    _stprintf(szSrcFile, __TEXT("%s\\%s"), g_szTargetDir, pszFile);

    if( !CreateHardLink(pszFile, szSrcFile, 0) )
    {
        ShowError();
    }
}

bool AskYesNo( TCHAR *pszPrompt )
{
    _tprintf(__TEXT("%s "), pszPrompt);

    int ch = getche();

    return bool(toupper(ch) == 'Y');
}

// int APIENTRY WinMain( HINSTANCE, HINSTANCE, LPSTR , int )
int _tmain( int cArgs, TCHAR *pszArgs[] )
{
    bool    fQuiet = false;
    TCHAR   szSearchSpec[64];

    if( !ParseCmdLine(pszArgs, cArgs, g_szTargetDir, szSearchSpec, &fQuiet) )
    {
        ShowUsage();
        return(0);
    }

    // Phase 1: build a list of files in OtherDir
    //
    WIN32_FIND_DATA FindData = {0};
    HANDLE          hFind;
    TCHAR           szFileSpec[MAX_PATH + 10];

    _stprintf(szFileSpec, __TEXT("%s\\%s"), g_szTargetDir, szSearchSpec);

    hFind = FindFirstFile(szFileSpec, &FindData);

    if( hFind!= INVALID_HANDLE_VALUE )
    {
        BOOL fMoreFiles;

        do
        {
            if( IsValidFile(&FindData) )
            {
                g_listFiles.push_back( new _tstring(FindData.cFileName) );
            }

            fMoreFiles = FindNextFile(hFind, &FindData);
        }
        while( fMoreFiles );

        FindClose(hFind);

        TCHAR szConfirm[128];

        _stprintf( szConfirm, __TEXT("Create links to %d files in %s?"),
                   g_listFiles.size(), g_szTargetDir );
        
        if( fQuiet || AskYesNo(szConfirm) )
        {
            // Phase 2: create links to each file in the other directory.
            //
            for_each( g_listFiles.begin(), g_listFiles.end(),
                      LinkToFile );
        }
    }
    else
    {
        ShowError();
    }

	return 0;
}



